/* Program koji broji pojavljivanja svih etiketa u HTML datoteci 
	- etikete se ispisuju opadajuci po broju pojavljivanja 
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

/* Struktura cvora drveta u kome se cuvaju etikete zajedno sa brojem
   pojavljivanja. Drvo je pretrazivacko i sortirano je lekikografski po 
   etiketama */
typedef struct _node{
	char tag[30];
	int num;
	struct _node *l,*r;
} node;


/* Zbog sortiranja po broju cvorova, paralelno sa strukturom drveta, 
   odrzavamo niz pokazivaca na njegove cvorove */
node* nodes[100];
/* Dosadasnji broj cvorova drveta (razlicitih etiketa) */
int num_nodes = 0;


/* Funkcija kreira cvor koji sadrzi datu etiketu */
node* make_node(char *tag)
{
	node* new_node = (node*) malloc(sizeof(node));
	if(new_node == NULL)
	{ 
		fprintf(stderr,"Greska prilikom alokacije memorije\n");
		exit(1);
	}  

	strcpy(new_node->tag, tag);
	new_node->l=NULL;
	new_node->r=NULL;
	new_node->num = 1;

	/* Dopisujemo cvor u niz postojecih cvorova */
	nodes[num_nodes++] = new_node;

	return new_node;
}

/* Funkcija umece datu etiketu u postojece drvo. Ukoliko etiketa postoji,
   povecava se njen broj pojavljivanja */
void insert(node** ptree, char tag[])
{
	int cmp;
	if(*ptree==NULL)
	{ 
		*ptree = make_node(tag);
		return;
	}
	
	cmp = strcmp(tag, (*ptree)->tag);
	
	if (cmp < 0)
		insert(&((*ptree)->l), tag);
	else if (cmp > 0)
		insert(&((*ptree)->r), tag);
	else
		(*ptree)->num++;
}

/* Funcija za ispis drveta */
void print(node* tree)
{
	if(tree != NULL)
	{
		print(tree->l);
		printf("%-10s - %3d\n", tree->tag, tree->num);
		print(tree->r);
	}
}

/* Funkcija koja uklanja drvo */ 
void remove_tree(node* tree)
{
	if(tree != NULL)
	{ 
		remove_tree(tree->l);
		remove_tree(tree->r);
		free(tree);
	}
}
 

/* Funkcija poredjenja za poziv ugradjene funkcije qsort. Porede se 
   dva cvora drveta na osnovu broja pojavljivanja etiketa */
int compare(const void* pa, const void* pb)
{
	return (*((node**)pb))->num - (*((node**)pa))->num;
}

/* Funkcija ucitava etiketu iz date datoteke. Funkcija vraca
   logicku vrednost koja indikuje da li je etiketa uspesno 
   procitana */
int get_tag(FILE* f, char tag[])
{
	int c;
	int i = 0;

	/* Preskacemo sve do prvog znaka '<' ili kraja */
	while ((c = fgetc(f)) != EOF && c != '<')
		;

	/* Nije bilo vise etiketa */
	if (c == EOF)
		return 0;
	
	/* Citamo prvi karakter etiketa*/
	c = fgetc(f);

	/* Gutamo / kod zatvorenih etiketa */
	if (c == '/')
		c = fgetc(f);

	/* Ime etikete cine slova */
	while(isalpha(c))
	{
		tag[i++] = c;
		c = fgetc(f);
	}
	tag[i] = '\0';

	/* Preskacemo sve do > ili do kraja */
	while (c != '>' && c != EOF)
		c = fgetc(f);

	if (c == EOF)
		return 0;

	return 1;
}

main(int argc, char* argv[])
{
	/* Tekuca procitana etiketa */
	char tag[30];

	/* Drvo koje je leksikografski sortirano zbog brze pretrage */
	node* tree = NULL;
	
	/* Datoteka iz koje se cita */
	FILE* f;
	int i;

	/* Proverava se korektnost argumenata komandne linije */
	if (argc < 2)
	{
		printf("Upotreba : %s ime_datoteke\n", argv[0]);
		exit(1);
	}
	
	/* Otvara se datoteka */
	f = fopen(argv[1], "r");
	if (f == NULL)
	{
		fprintf(stderr, "Greska prilikom otvaranja %s\n", argv[1]);
		return;
	}

	/* Kreiramo drvo na osnovu sadrzaja datoteke */
	while(get_tag(f, tag) == 1)
		insert(&tree, tag);

	/* Zatvaramo datoteku */
	fclose(f);

	/* Sortiramo cvorove niza na osnovu broja pojavljivanja etiketa */
	qsort(nodes, num_nodes, sizeof(node*), &compare);

	/* Ispisujemo sortirane cvorove */
	for (i = 0; i<num_nodes; i++)
		printf("%-10s - %3d\n", nodes[i]->tag, nodes[i]->num);

	/* Uklanjamo drvo */	
	remove_tree(tree);

}

